@@ -35,6 +35,10 @@ class MatchingResults(ImputerResults):
         seed: int,
         imputed_vars_dummy_info: Optional[Dict[str, Any]] = None,
         original_predictors: Optional[List[str]] = None,
+        categorical_targets: Optional[Dict[str, Dict]] = None,
+        boolean_targets: Optional[Dict[str, Dict]] = None,
+        constant_targets: Optional[Dict[str, Dict]] = None,
+        dummy_processor: Optional[Any] = None,
         log_level: Optional[str] = "WARNING",
         hyperparameters: Optional[Dict[str, Any]] = None,
     ) -> None:
@@ -50,6 +54,9 @@ class MatchingResults(ImputerResults):
                 about dummy variables for imputed variables.
             original_predictors: Optional list of original predictor names
                 before dummy encoding.
+            categorical_targets: Dictionary of categorical target info.
+            boolean_targets: Dictionary of boolean target info.
+            dummy_processor: Processor for handling dummy encoding in test data.
             hyperparameters: Optional dictionary of hyperparameters for the
                 matching function, specified after tunning.
         """
@@ -64,19 +71,27 @@ class MatchingResults(ImputerResults):
         self.matching_hotdeck = matching_hotdeck
         self.donor_data = donor_data
         self.hyperparameters = hyperparameters
+        self.categorical_targets = categorical_targets or {}
+        self.boolean_targets = boolean_targets or {}
+        self.dummy_processor = dummy_processor
 
     @validate_call(config=VALIDATE_CONFIG)
     def _predict(
-        self, X_test: pd.DataFrame, quantiles: Optional[List[float]] = None
+        self,
+        X_test: pd.DataFrame,
+        quantiles: Optional[List[float]] = None,
+        return_probs: bool = False,
     ) -> Dict[float, pd.DataFrame]:
         """Predict imputed values using the matching model.
 
         Args:
             X_test: DataFrame containing the recipient data.
             quantiles: List of quantiles to predict.
+            return_probs: If True, return one-hot probability vectors for matched categories.
 
         Returns:
             Dictionary mapping quantiles to imputed values.
+            If return_probs=True, includes 'probabilities' key with one-hot encodings.
 
         Raises:
             ValueError: If model is not properly set up or
@@ -128,10 +143,12 @@ class MatchingResults(ImputerResults):
                     f"{len(self.donor_data)} donor records). Using chunking approach."
                 )
                 return self._predict_chunked(
-                    X_test_copy, quantiles, chunk_size
+                    X_test_copy, quantiles, chunk_size, return_probs
                 )
             else:
-                return self._predict_single(X_test_copy, quantiles)
+                return self._predict_single(
+                    X_test_copy, quantiles, return_probs
+                )
 
         except ValueError as e:
             raise e
@@ -143,6 +160,7 @@ class MatchingResults(ImputerResults):
         self,
         X_test_copy: pd.DataFrame,
         quantiles: Optional[List[float]] = None,
+        return_probs: bool = False,
     ) -> Dict[float, pd.DataFrame]:
         """Perform matching on the full dataset without chunking."""
         try:
@@ -168,13 +186,16 @@ class MatchingResults(ImputerResults):
             )
             raise RuntimeError("Hot deck matching failed") from matching_error
 
-        return self._process_matching_results(fused0, X_test_copy, quantiles)
+        return self._process_matching_results(
+            fused0, X_test_copy, quantiles, return_probs
+        )
 
     def _predict_chunked(
         self,
         X_test_copy: pd.DataFrame,
         quantiles: Optional[List[float]],
         chunk_size: int,
+        return_probs: bool = False,
     ) -> Dict[float, pd.DataFrame]:
         """Perform matching using chunking for large datasets."""
         all_results = []
@@ -231,16 +252,68 @@ class MatchingResults(ImputerResults):
             combined_results = combined_results.loc[X_test_copy.index]
 
             return self._process_matching_results(
-                combined_results, X_test_copy, quantiles
+                combined_results, X_test_copy, quantiles, return_probs
             )
         else:
             raise RuntimeError("No chunk results were produced")
 
+    def _generate_one_hot_probabilities(
+        self,
+        variable: str,
+        matched_values: np.ndarray,
+        index: pd.Index,
+        categorical_targets: Dict,
+        boolean_targets: Dict,
+    ) -> Optional[Dict]:
+        """Generate one-hot probability matrix for categorical/boolean variables.
+
+        Args:
+            variable: Name of the variable
+            matched_values: Array of matched category values
+            index: Index for the output DataFrame
+            categorical_targets: Dictionary of categorical target info
+            boolean_targets: Dictionary of boolean target info
+
+        Returns:
+            Dict with 'probabilities' and 'classes' keys
+        """
+        if (
+            variable not in categorical_targets
+            and variable not in boolean_targets
+        ):
+            return None
+
+        # Determine categories
+        if variable in boolean_targets:
+            categories = [False, True]
+        else:
+            categories = categorical_targets[variable].get("categories", [])
+
+        if not categories:
+            return None
+
+        # Create probability matrix (one-hot encoding)
+        n_samples = len(matched_values)
+        n_categories = len(categories)
+        prob_matrix = np.zeros((n_samples, n_categories))
+
+        # Set 1.0 for matched category
+        for idx, val in enumerate(matched_values):
+            try:
+                cat_idx = categories.index(val)
+                prob_matrix[idx, cat_idx] = 1.0
+            except ValueError:
+                # If value not found in categories, default to first category
+                prob_matrix[idx, 0] = 1.0
+
+        return {"probabilities": prob_matrix, "classes": np.array(categories)}
+
     def _process_matching_results(
         self,
         fused0: pd.DataFrame,
         X_test_copy: pd.DataFrame,
         quantiles: Optional[List[float]],
+        return_probs: bool = False,
     ) -> Dict[float, pd.DataFrame]:
         """Process matching results into the expected output format."""
         try:
@@ -271,6 +344,11 @@ class MatchingResults(ImputerResults):
 
         # Create output dictionary with results
         imputations: Dict[float, pd.DataFrame] = {}
+        prob_results = {} if return_probs else None
+
+        # Get target type information if available
+        categorical_targets = getattr(self, "categorical_targets", {})
+        boolean_targets = getattr(self, "boolean_targets", {})
 
         try:
             if quantiles:
@@ -286,7 +364,24 @@ class MatchingResults(ImputerResults):
                         )
                         imputed_df[variable] = fused0[variable].values
 
+                        # Generate one-hot probabilities if requested
+                        if return_probs and prob_results is not None:
+                            prob_df = self._generate_one_hot_probabilities(
+                                variable,
+                                fused0[variable].values,
+                                X_test_copy.index,
+                                categorical_targets,
+                                boolean_targets,
+                            )
+                            if prob_df is not None:
+                                prob_results[variable] = prob_df
+
                     imputations[q] = imputed_df
+
+                # Add probabilities to results if requested
+                if return_probs and prob_results:
+                    imputations["probabilities"] = prob_results
+
                 return imputations
             else:
                 # If no quantiles specified, use a default one
@@ -298,9 +393,29 @@ class MatchingResults(ImputerResults):
                 for variable in self.imputed_variables:
                     self.logger.info(f"Imputing variable {variable}")
                     imputed_df[variable] = fused0[variable].values
+
+                    # Generate one-hot probabilities if requested
+                    if return_probs and prob_results is not None:
+                        prob_df = self._generate_one_hot_probabilities(
+                            variable,
+                            fused0[variable].values,
+                            X_test_copy.index,
+                            categorical_targets,
+                            boolean_targets,
+                        )
+                        if prob_df is not None:
+                            prob_results[variable] = prob_df
+
                 imputations[q_default] = imputed_df
 
-                return imputations[q_default]
+                # Add probabilities to results if requested
+                if return_probs and prob_results:
+                    # Return dict with both quantile predictions and probabilities
+                    imputations["probabilities"] = prob_results
+                    return imputations
+                else:
+                    # Return just the DataFrame for the single quantile
+                    return imputations[q_default]
         except Exception as output_error:
             self.logger.error(
                 f"Error creating output imputations: {str(output_error)}"
@@ -352,6 +467,10 @@ class Matching(Imputer):
         predictors: List[str],
         imputed_variables: List[str],
         original_predictors: Optional[List[str]] = None,
+        categorical_targets: Optional[Dict[str, Dict]] = None,
+        boolean_targets: Optional[Dict[str, Dict]] = None,
+        numeric_targets: Optional[List[str]] = None,
+        constant_targets: Optional[Dict[str, Dict]] = None,
         tune_hyperparameters: bool = False,
         **matching_kwargs: Any,
     ) -> MatchingResults:
@@ -415,6 +534,9 @@ class Matching(Imputer):
                     imputed_variables=imputed_variables,
                     imputed_vars_dummy_info=self.imputed_vars_dummy_info,
                     original_predictors=self.original_predictors,
+                    categorical_targets=categorical_targets,
+                    boolean_targets=boolean_targets,
+                    dummy_processor=getattr(self, "dummy_processor", None),
                     seed=self.seed,
                     log_level=self.log_level,
                     hyperparameters=matching_kwargs,
@@ -432,10 +554,12 @@ class Matching(Imputer):
         predictors: List[str],
         imputed_variables: List[str],
     ) -> Dict[str, Any]:
-        """Tune hyperparameters for the Matching model using Optuna.
+        """Tune hyperparameters for the Matching model using Optuna with CV.
+
+        Uses cross-validation and quantile loss for robust hyperparameter selection.
 
         Args:
-            X_train: DataFrame containing the training data.
+            data: DataFrame containing the training data.
             predictors: List of column names to use as predictors.
             imputed_variables: List of column names to impute.
 
@@ -443,13 +567,21 @@ class Matching(Imputer):
             Dictionary of tuned hyperparameters.
         """
         import optuna
-        from sklearn.model_selection import train_test_split
+        from sklearn.model_selection import KFold
+
+        from microimpute.comparisons.metrics import compute_loss
 
         optuna.logging.set_verbosity(optuna.logging.WARNING)
 
-        # Create a validation split (80% train, 20% validation)
-        X_train, X_test = train_test_split(
-            data, test_size=0.2, random_state=self.seed
+        # Use same CV strategy as QRF: 3-fold CV with 10 trials
+        n_cv_folds = 3
+        n_trials = 10
+
+        # Set up CV folds
+        kf = KFold(n_splits=n_cv_folds, shuffle=True, random_state=self.seed)
+
+        self.logger.info(
+            f"Tuning Matching hyperparameters with {n_cv_folds}-fold CV and {n_trials} trials"
         )
 
         def objective(trial: optuna.Trial) -> float:
@@ -473,81 +605,103 @@ class Matching(Imputer):
                 "k": trial.suggest_int("k", 1, 10),
             }
 
-            # Track errors for all variables
-            var_errors = []
+            # Track errors across CV folds
+            fold_errors = []
 
-            for var in imputed_variables:
-                y_test = X_test[var]
-                X_test_var = X_test.copy().drop(var, axis=1)
+            # Perform CV
+            for fold_idx, (train_idx, val_idx) in enumerate(kf.split(data)):
+                X_train_fold = data.iloc[train_idx]
+                X_val_fold = data.iloc[val_idx]
 
-                # Determine if chunking is needed for hyperparameter tuning
-                chunk_size = 1000  # Smaller chunks for tuning
-                total_size = len(X_train) * len(X_test_var)
-                use_chunking = (
-                    len(X_test_var) > chunk_size
-                    or total_size > 25_000_000  # Lower threshold for tuning
-                )
+                # Track errors for all variables in this fold
+                var_errors = []
 
-                if use_chunking:
-                    # Perform chunked matching for hyperparameter tuning
-                    y_pred_chunks = []
-                    y_test_chunks = []
+                for var in imputed_variables:
+                    y_val = X_val_fold[var]
+                    X_val_var = X_val_fold.copy().drop(var, axis=1)
 
-                    for i in range(0, len(X_test_var), chunk_size):
-                        chunk_end = min(i + chunk_size, len(X_test_var))
-                        chunk_data = X_test_var.iloc[i:chunk_end]
-                        chunk_y_test = y_test.iloc[i:chunk_end]
+                    # Determine if chunking is needed for hyperparameter tuning
+                    chunk_size = 1000  # Smaller chunks for tuning
+                    total_size = len(X_train_fold) * len(X_val_var)
+                    use_chunking = (
+                        len(X_val_var) > chunk_size
+                        or total_size
+                        > 25_000_000  # Lower threshold for tuning
+                    )
 
+                    if use_chunking:
+                        # Perform chunked matching for hyperparameter tuning
+                        y_pred_chunks = []
+                        y_val_chunks = []
+
+                        for i in range(0, len(X_val_var), chunk_size):
+                            chunk_end = min(i + chunk_size, len(X_val_var))
+                            chunk_data = X_val_var.iloc[i:chunk_end]
+                            chunk_y_val = y_val.iloc[i:chunk_end]
+
+                            try:
+                                fused0, fused1 = self.matching_hotdeck(
+                                    receiver=chunk_data,
+                                    donor=X_train_fold,
+                                    matching_variables=predictors,
+                                    z_variables=[var],
+                                    **params,
+                                )
+                                y_pred_chunks.append(fused0[var].values)
+                                y_val_chunks.append(chunk_y_val.values)
+                            except Exception:
+                                # If chunk fails, use mean of training data as prediction
+                                mean_val = X_train_fold[var].mean()
+                                y_pred_chunks.append(
+                                    np.full(len(chunk_data), mean_val)
+                                )
+                                y_val_chunks.append(chunk_y_val.values)
+
+                        # Combine chunk results
+                        y_pred = np.concatenate(y_pred_chunks)
+                        y_val_combined = np.concatenate(y_val_chunks)
+                    else:
+                        # Perform single matching
                         try:
                             fused0, fused1 = self.matching_hotdeck(
-                                receiver=chunk_data,
-                                donor=X_train,
+                                receiver=X_val_var,
+                                donor=X_train_fold,
                                 matching_variables=predictors,
                                 z_variables=[var],
                                 **params,
                             )
-                            y_pred_chunks.append(fused0[var].values)
-                            y_test_chunks.append(chunk_y_test.values)
+                            y_pred = fused0[var].values
+                            y_val_combined = y_val.values
                         except Exception:
-                            # If chunk fails, use mean of training data as prediction
-                            mean_val = X_train[var].mean()
-                            y_pred_chunks.append(
-                                np.full(len(chunk_data), mean_val)
-                            )
-                            y_test_chunks.append(chunk_y_test.values)
+                            # If matching fails, use mean of training data as prediction
+                            mean_val = X_train_fold[var].mean()
+                            y_pred = np.full(len(X_val_var), mean_val)
+                            y_val_combined = y_val.values
 
-                    # Combine chunk results
-                    y_pred = np.concatenate(y_pred_chunks)
-                    y_test_combined = np.concatenate(y_test_chunks)
-                else:
-                    # Perform single matching
-                    try:
-                        fused0, fused1 = self.matching_hotdeck(
-                            receiver=X_test_var,
-                            donor=X_train,
-                            matching_variables=predictors,
-                            z_variables=[var],
-                            **params,
-                        )
-                        y_pred = fused0[var].values
-                        y_test_combined = y_test.values
-                    except Exception:
-                        # If matching fails, use mean of training data as prediction
-                        mean_val = X_train[var].mean()
-                        y_pred = np.full(len(X_test_var), mean_val)
-                        y_test_combined = y_test.values
+                    # Use quantile loss with median (q=0.5) for hyperparameter tuning
+                    _, quantile_loss_value = compute_loss(
+                        y_val_combined.flatten(),
+                        y_pred.flatten(),
+                        "quantile_loss",
+                        q=0.5,
+                    )
 
-                # Calculate error
-                # Normalize error by variable's standard deviation
-                std = np.std(y_test_combined.flatten())
-                mse = np.mean(
-                    (y_pred.flatten() - y_test_combined.flatten()) ** 2
-                )
-                normalized_mse = mse / (std**2) if std > 0 else mse
+                    # Normalize by variable's standard deviation
+                    std = np.std(y_val_combined.flatten())
+                    normalized_loss = (
+                        quantile_loss_value / std
+                        if std > 0
+                        else quantile_loss_value
+                    )
 
-                var_errors.append(normalized_mse)
+                    var_errors.append(normalized_loss)
 
-            return np.mean(var_errors)
+                # Average across variables for this fold
+                if var_errors:
+                    fold_errors.append(np.mean(var_errors))
+
+            # Return mean error across all CV folds
+            return np.mean(fold_errors) if fold_errors else float("inf")
 
         study = optuna.create_study(
             direction="minimize",
@@ -559,12 +713,16 @@ class Matching(Imputer):
 
         os.environ["PYTHONWARNINGS"] = "ignore"
 
-        study.optimize(objective, n_trials=30)
+        study.optimize(objective, n_trials=n_trials)
 
         best_value = study.best_value
-        self.logger.info(f"Lowest average normalized MSE: {best_value}")
+        self.logger.info(
+            f"Matching - Lowest average normalized quantile loss ({n_cv_folds}-fold CV): {best_value}"
+        )
 
         best_params = study.best_params
-        self.logger.info(f"Best hyperparameters found: {best_params}")
+        self.logger.info(
+            f"Matching - Best hyperparameters found: {best_params}"
+        )
 
         return best_params
